@@ -27,6 +27,7 @@
 #ifndef RNP_FILE_UTILS_H_
 #define RNP_FILE_UTILS_H_
 
+#include "config.h"
 #include <stdint.h>
 #include <stdio.h>
 #include <dirent.h>
@@ -39,9 +40,13 @@ int     rnp_open(const char *filename, int oflag, int pmode);
 FILE *  rnp_fopen(const char *filename, const char *mode);
 FILE *  rnp_fdopen(int fildes, const char *mode);
 int     rnp_access(const char *path, int mode);
-int     rnp_stat(const char *filename, struct stat *statbuf);
-int     rnp_rename(const char *oldpath, const char *newpath);
-int     rnp_unlink(const char *path);
+#if defined(_WIN32) && !defined(HAVE_WIN_STAT)
+/* Default to __stat64 structure unless defined by mingw since commit [07e345] */
+#define stat __stat64
+#endif
+int rnp_stat(const char *filename, struct stat *statbuf);
+int rnp_rename(const char *oldpath, const char *newpath);
+int rnp_unlink(const char *path);
 
 #ifdef _WIN32
 #define rnp_closedir _wclosedir
