@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2017 Ribose Inc.
+ * Copyright (c) 2017-2024 Ribose Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,20 +29,22 @@
 
 #include "crypto/ec.h"
 
-rnp_result_t ecdsa_validate_key(rnp::RNG *rng, const pgp_ec_key_t *key, bool secret);
+namespace pgp {
+namespace ecdsa {
+rnp_result_t validate_key(rnp::RNG &rng, const ec::Key &key, bool secret);
 
-rnp_result_t ecdsa_sign(rnp::RNG *          rng,
-                        pgp_ec_signature_t *sig,
-                        pgp_hash_alg_t      hash_alg,
-                        const uint8_t *     hash,
-                        size_t              hash_len,
-                        const pgp_ec_key_t *key);
+rnp_result_t sign(rnp::RNG &               rng,
+                  ec::Signature &          sig,
+                  pgp_hash_alg_t           hash_alg,
+                  const rnp::secure_bytes &hash,
+                  const ec::Key &          key);
 
-rnp_result_t ecdsa_verify(const pgp_ec_signature_t *sig,
-                          pgp_hash_alg_t            hash_alg,
-                          const uint8_t *           hash,
-                          size_t                    hash_len,
-                          const pgp_ec_key_t *      key);
+rnp_result_t verify(const ec::Signature &    sig,
+                    pgp_hash_alg_t           hash_alg,
+                    const rnp::secure_bytes &hash,
+                    const ec::Key &          key);
+
+const char *padding_str_for(pgp_hash_alg_t hash_alg);
 
 /*
  * @brief   Returns hash which should be used with the curve
@@ -52,6 +54,9 @@ rnp_result_t ecdsa_verify(const pgp_ec_signature_t *sig,
  * @returns  Either ID of the hash algorithm, or PGP_HASH_UNKNOWN
  *           if not found
  */
-pgp_hash_alg_t ecdsa_get_min_hash(pgp_curve_t curve);
+pgp_hash_alg_t get_min_hash(pgp_curve_t curve);
+
+} // namespace ecdsa
+} // namespace pgp
 
 #endif // ECDSA_H_
