@@ -37,7 +37,7 @@ typedef std::shared_ptr<gnupg_sexp_t> p_gnupg_sexp;
 
 class gnupg_sexp_t : public sexp::sexp_list_t {
     /* write gnupg_sexp_t contents, adding padding, for the further encryption */
-    rnp::secure_vector<uint8_t> write_padded(size_t padblock) const;
+    rnp::secure_bytes write_padded(size_t padblock) const;
 
   public:
     void
@@ -52,8 +52,8 @@ class gnupg_sexp_t : public sexp::sexp_list_t {
     };
     void         add(unsigned u);
     p_gnupg_sexp add_sub();
-    void         add_mpi(const std::string &name, const pgp_mpi_t &val);
-    void         add_curve(const std::string &name, const pgp_ec_key_t &key);
+    void         add_mpi(const std::string &name, const pgp::mpi &val);
+    void         add_curve(const std::string &name, pgp_curve_t curve);
     void         add_pubkey(const pgp_key_pkt_t &key);
     void         add_seckey(const pgp_key_pkt_t &key);
     void         add_protected_seckey(pgp_key_pkt_t &       seckey,
