@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2020 [Ribose Inc](https://www.ribose.com).
+ * Copyright (c) 2017-2020, 2023 [Ribose Inc](https://www.ribose.com).
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification,
@@ -45,6 +45,7 @@
 #include <rnp/rnp_def.h>
 #include "rnp.h"
 #include "stream-common.h"
+#include "stream-packet.h"
 #include "types.h"
 #include "file-utils.h"
 #include "crypto/mem.h"
@@ -52,25 +53,24 @@
 #include <memory>
 
 bool
-src_read(pgp_source_t *src, void *buf, size_t len, size_t *readres)
+pgp_source_t::read(void *buf, size_t len, size_t *readres)
 {
-    size_t              left = len;
-    size_t              read;
-    pgp_source_cache_t *cache = src->cache;
-    bool                readahead = cache ? cache->readahead : false;
+    size_t left = len;
+    size_t read;
+    bool   readahead = cache ? cache->readahead : false;
 
-    if (src->error) {
+    if (error_) {
         return false;
     }
 
-    if (src->eof || (len == 0)) {
+    if (eof_ || (len == 0)) {
         *readres = 0;
         return true;
     }
 
     // Do not read more then available if source size is known
-    if (src->knownsize && (src->readb + len > src->size)) {
-        len = src->size - src->readb;
+    if (knownsize && (readb + len > size)) {
+        len = size - readb;
         left = len;
         readahead = false;
     }
@@ -94,12 +94,12 @@ src_read(pgp_source_t *src, void *buf, size_t len, size_t *readres)
     while (left > 0) {
         if (left > sizeof(cache->buf) || !readahead || !cache) {
             // If there is no cache or chunk is larger then read directly
-            if (!src->read(src, buf, left, &read)) {
-                src->error = 1;
+            if (!raw_read(this, buf, left, &read)) {
+                error_ = 1;
                 return false;
             }
             if (!read) {
-                src->eof = 1;
+                eof_ = true;
                 len = len - left;
                 goto finish;
             }
@@ -107,12 +107,12 @@ src_read(pgp_source_t *src, void *buf, size_t len, size_t *readres)
             buf = (uint8_t *) buf + read;
         } else {
             // Try to fill the cache to avoid small reads
-            if (!src->read(src, &cache->buf[0], sizeof(cache->buf), &read)) {
-                src->error = 1;
+            if (!raw_read(this, &cache->buf[0], sizeof(cache->buf), &read)) {
+                error_ = true;
                 return false;
             }
             if (!read) {
-                src->eof = 1;
+                eof_ = true;
                 len = len - left;
                 goto finish;
             } else if (read < left) {
@@ -129,32 +129,31 @@ src_read(pgp_source_t *src, void *buf, size_t len, size_t *readres)
     }
 
 finish:
-    src->readb += len;
-    if (src->knownsize && (src->readb == src->size)) {
-        src->eof = 1;
+    readb += len;
+    if (knownsize && (readb == size)) {
+        eof_ = true;
     }
     *readres = len;
     return true;
 }
 
 bool
-src_read_eq(pgp_source_t *src, void *buf, size_t len)
+pgp_source_t::read_eq(void *buf, size_t len)
 {
     size_t res = 0;
-    return src_read(src, buf, len, &res) && (res == len);
+    return read(buf, len, &res) && (res == len);
 }
 
 bool
-src_peek(pgp_source_t *src, void *buf, size_t len, size_t *peeked)
+pgp_source_t::peek(void *buf, size_t len, size_t *peeked)
 {
-    pgp_source_cache_t *cache = src->cache;
-    if (src->error) {
+    if (error_) {
         return false;
     }
     if (!cache || (len > sizeof(cache->buf))) {
         return false;
     }
-    if (src->eof) {
+    if (eof_) {
         *peeked = 0;
         return true;
     }
@@ -162,8 +161,8 @@ src_peek(pgp_source_t *src, void *buf, size_t len, size_t *peeked)
     size_t read = 0;
     bool   readahead = cache->readahead;
     // Do not read more then available if source size is known
-    if (src->knownsize && (src->readb + len > src->size)) {
-        len = src->size - src->readb;
+    if (knownsize && (readb + len > size)) {
+        len = size - readb;
         readahead = false;
     }
 
@@ -183,11 +182,11 @@ src_peek(pgp_source_t *src, void *buf, size_t len, size_t *peeked)
 
     while (cache->len < len) {
         read = readahead ? sizeof(cache->buf) - cache->len : len - cache->len;
-        if (src->knownsize && (src->readb + read > src->size)) {
-            read = src->size - src->readb;
+        if (knownsize && (readb + read > size)) {
+            read = size - readb;
         }
-        if (!src->read(src, &cache->buf[cache->len], read, &read)) {
-            src->error = 1;
+        if (!raw_read(this, &cache->buf[cache->len], read, &read)) {
+            error_ = true;
             return false;
         }
         if (!read) {
@@ -210,39 +209,39 @@ src_peek(pgp_source_t *src, void *buf, size_t len, size_t *peeked)
 }
 
 bool
-src_peek_eq(pgp_source_t *src, void *buf, size_t len)
+pgp_source_t::peek_eq(void *buf, size_t len)
 {
     size_t res = 0;
-    return src_peek(src, buf, len, &res) && (res == len);
+    return peek(buf, len, &res) && (res == len);
 }
 
 void
-src_skip(pgp_source_t *src, size_t len)
+pgp_source_t::skip(size_t len)
 {
-    if (src->cache && (src->cache->len - src->cache->pos >= len)) {
-        src->readb += len;
-        src->cache->pos += len;
+    if (cache && (cache->len - cache->pos >= len)) {
+        readb += len;
+        cache->pos += len;
         return;
     }
 
     size_t  res = 0;
     uint8_t sbuf[16];
     if (len < sizeof(sbuf)) {
-        (void) src_read(src, sbuf, len, &res);
+        (void) read(sbuf, len, &res);
         return;
     }
-    if (src_eof(src)) {
+    if (eof()) {
         return;
     }
 
     void *buf = calloc(1, std::min((size_t) PGP_INPUT_CACHE_SIZE, len));
     if (!buf) {
-        src->error = 1;
+        error_ = true;
         return;
     }
 
-    while (len && !src_eof(src)) {
-        if (!src_read(src, buf, std::min((size_t) PGP_INPUT_CACHE_SIZE, len), &res)) {
+    while (len && !eof()) {
+        if (!read(buf, std::min((size_t) PGP_INPUT_CACHE_SIZE, len), &res)) {
             break;
         }
         len -= res;
@@ -251,69 +250,87 @@ src_skip(pgp_source_t *src, size_t len)
 }
 
 rnp_result_t
-src_finish(pgp_source_t *src)
+pgp_source_t::finish()
 {
-    rnp_result_t res = RNP_SUCCESS;
-    if (src->finish) {
-        res = src->finish(src);
+    if (raw_finish) {
+        return raw_finish(this);
     }
-
-    return res;
+    return RNP_SUCCESS;
 }
 
 bool
-src_error(const pgp_source_t *src)
+pgp_source_t::error() const
 {
-    return src->error;
+    return error_;
 }
 
 bool
-src_eof(pgp_source_t *src)
+pgp_source_t::eof()
 {
-    if (src->eof) {
+    if (eof_) {
         return true;
     }
-    /* Error on stream read is NOT considered as eof. See src_error(). */
+    /* Error on stream read is NOT considered as eof. See error(). */
     uint8_t check;
     size_t  read = 0;
-    return src_peek(src, &check, 1, &read) && (read == 0);
+    return peek(&check, 1, &read) && (read == 0);
 }
 
 void
-src_close(pgp_source_t *src)
+pgp_source_t::close()
 {
-    if (src->close) {
-        src->close(src);
+    if (raw_close) {
+        raw_close(this);
     }
 
-    if (src->cache) {
-        free(src->cache);
-        src->cache = NULL;
+    if (cache) {
+        free(cache);
+        cache = NULL;
     }
 }
 
 bool
-src_skip_eol(pgp_source_t *src)
+pgp_source_t::skip_eol()
 {
     uint8_t eol[2];
     size_t  read;
 
-    if (!src_peek(src, eol, 2, &read) || !read) {
+    if (!peek(eol, 2, &read) || !read) {
         return false;
     }
     if (eol[0] == '\n') {
-        src_skip(src, 1);
+        skip(1);
         return true;
     }
     if ((read == 2) && (eol[0] == '\r') && (eol[1] == '\n')) {
-        src_skip(src, 2);
+        skip(2);
         return true;
     }
     return false;
 }
 
 bool
-src_peek_line(pgp_source_t *src, char *buf, size_t len, size_t *readres)
+pgp_source_t::skip_chars(const std::string &chars)
+{
+    do {
+        char   ch = 0;
+        size_t read = 0;
+        if (!peek(&ch, 1, &read)) {
+            return false;
+        }
+        if (!read) {
+            /* return true only if there is no underlying read error */
+            return true;
+        }
+        if (chars.find(ch) == std::string::npos) {
+            return true;
+        }
+        skip(1);
+    } while (1);
+}
+
+bool
+pgp_source_t::peek_line(char *buf, size_t len, size_t *readres)
 {
     size_t scan_pos = 0;
     size_t inc = 64;
@@ -325,7 +342,7 @@ src_peek_line(pgp_source_t *src, char *buf, size_t len, size_t *readres)
         inc = inc * 2;
 
         /* inefficient, each time we again read from the beginning */
-        if (!src_peek(src, buf, to_peek, readres)) {
+        if (!peek(buf, to_peek, readres)) {
             return false;
         }
 
@@ -412,8 +429,8 @@ init_fd_src(pgp_source_t *src, int fd, uint64_t *size)
 
     pgp_source_file_param_t *param = (pgp_source_file_param_t *) src->param;
     param->fd = fd;
-    src->read = file_src_read;
-    src->close = file_src_close;
+    src->raw_read = file_src_read;
+    src->raw_close = file_src_close;
     src->type = PGP_STREAM_FILE;
     src->size = size ? *size : 0;
     src->knownsize = !!size;
@@ -471,8 +488,8 @@ init_stdin_src(pgp_source_t *src)
 
     param = (pgp_source_file_param_t *) src->param;
     param->fd = 0;
-    src->read = file_src_read;
-    src->close = file_src_close;
+    src->raw_read = file_src_read;
+    src->raw_close = file_src_close;
     src->type = PGP_STREAM_STDIN;
 
     return RNP_SUCCESS;
@@ -540,9 +557,9 @@ init_mem_src(pgp_source_t *src, const void *mem, size_t len, bool free)
     param->len = len;
     param->pos = 0;
     param->free = free;
-    src->read = mem_src_read;
-    src->close = mem_src_close;
-    src->finish = NULL;
+    src->raw_read = mem_src_read;
+    src->raw_close = mem_src_close;
+    src->raw_finish = NULL;
     src->size = len;
     src->knownsize = 1;
     src->type = PGP_STREAM_MEMORY;
@@ -560,9 +577,9 @@ rnp_result_t
 init_null_src(pgp_source_t *src)
 {
     memset(src, 0, sizeof(*src));
-    src->read = null_src_read;
+    src->raw_read = null_src_read;
     src->type = PGP_STREAM_NULL;
-    src->error = true;
+    src->error_ = true;
     return RNP_SUCCESS;
 }
 
@@ -570,7 +587,8 @@ rnp_result_t
 read_mem_src(pgp_source_t *src, pgp_source_t *readsrc)
 {
     pgp_dest_t   dst;
-    rnp_result_t ret;
+    rnp_result_t ret = RNP_ERROR_GENERIC;
+    size_t       sz = 0;
 
     if ((ret = init_mem_dest(&dst, NULL, 0))) {
         return ret;
@@ -580,32 +598,13 @@ read_mem_src(pgp_source_t *src, pgp_source_t *readsrc)
         goto done;
     }
 
-    if ((ret = init_mem_src(src, mem_dest_own_memory(&dst), dst.writeb, true))) {
-        goto done;
-    }
-
-    ret = RNP_SUCCESS;
+    sz = dst.writeb;
+    ret = init_mem_src(src, mem_dest_own_memory(&dst), sz, true);
 done:
     dst_close(&dst, true);
     return ret;
 }
 
-rnp_result_t
-file_to_mem_src(pgp_source_t *src, const char *filename)
-{
-    pgp_source_t fsrc = {};
-    rnp_result_t res = RNP_ERROR_GENERIC;
-
-    if ((res = init_file_src(&fsrc, filename))) {
-        return res;
-    }
-
-    res = read_mem_src(src, &fsrc);
-    src_close(&fsrc);
-
-    return res;
-}
-
 const void *
 mem_src_get_memory(pgp_source_t *src, bool own)
 {
@@ -673,7 +672,13 @@ dst_write(pgp_dest_t *dst, const void *buf, size_t len)
 }
 
 void
-dst_printf(pgp_dest_t *dst, const char *format, ...)
+dst_write(pgp_dest_t &dst, const std::vector<uint8_t> &buf)
+{
+    dst_write(&dst, buf.data(), buf.size());
+}
+
+void
+dst_printf(pgp_dest_t &dst, const char *format, ...)
 {
     char    buf[2048];
     size_t  len;
@@ -687,7 +692,7 @@ dst_printf(pgp_dest_t *dst, const char *format, ...)
         RNP_LOG("too long dst_printf");
         len = sizeof(buf) - 1;
     }
-    dst_write(dst, buf, len);
+    dst_write(&dst, buf, len);
 }
 
 void
@@ -1177,8 +1182,8 @@ dst_write_src(pgp_source_t *src, pgp_dest_t *dst, uint64_t limit)
         size_t   read;
         uint64_t totalread = 0;
 
-        while (!src->eof) {
-            if (!src_read(src, readbuf, bufsize, &read)) {
+        while (!src->eof_) {
+            if (!src->read(readbuf, bufsize, &read)) {
                 res = RNP_ERROR_GENERIC;
                 break;
             }
@@ -1210,3 +1215,60 @@ dst_write_src(pgp_source_t *src, pgp_dest_t *dst, uint64_t limit)
     dst_flush(dst);
     return dst->werr;
 }
+
+bool
+have_pkesk_checksum(pgp_pubkey_alg_t alg)
+{
+    switch (alg) {
+#if defined(ENABLE_CRYPTO_REFRESH) || defined(ENABLE_PQC)
+#if defined(ENABLE_CRYPTO_REFRESH)
+    case PGP_PKA_X25519:
+#endif
+#if defined(ENABLE_PQC)
+    case PGP_PKA_KYBER768_X25519:
+    // case PGP_PKA_KYBER1024_X448:
+    case PGP_PKA_KYBER768_P256:
+    case PGP_PKA_KYBER1024_P384:
+    case PGP_PKA_KYBER768_BP256:
+    case PGP_PKA_KYBER1024_BP384:
+#endif
+        return false;
+#endif
+    default:
+        return true;
+    }
+}
+
+bool
+do_encrypt_pkesk_v3_alg_id(pgp_pubkey_alg_t alg)
+{
+    /* matches the same algorithms */
+    return have_pkesk_checksum(alg);
+}
+
+#if defined(ENABLE_CRYPTO_REFRESH) || defined(ENABLE_PQC)
+/* The crypto refresh mandates that for a X25519/X448 PKESKv3, AES MUST be used.
+   The same is true for the PQC algorithms
+ */
+bool
+check_enforce_aes_v3_pkesk(pgp_pubkey_alg_t alg, pgp_symm_alg_t salg, pgp_pkesk_version_t ver)
+{
+    /* The same algorithms as with pkesk_checksum */
+    return (ver != PGP_PKSK_V3) || have_pkesk_checksum(alg) || pgp_is_sa_aes(salg);
+}
+#endif
+
+#if defined(ENABLE_AEAD)
+bool
+encrypted_sesk_set_ad(pgp_crypt_t &crypt, pgp_sk_sesskey_t &skey)
+{
+    uint8_t ad_data[4];
+
+    ad_data[0] = PGP_PKT_SK_SESSION_KEY | PGP_PTAG_ALWAYS_SET | PGP_PTAG_NEW_FORMAT;
+    ad_data[1] = skey.version;
+    ad_data[2] = skey.alg;
+    ad_data[3] = skey.aalg;
+
+    return pgp_cipher_aead_set_ad(&crypt, ad_data, 4);
+}
+#endif
