@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2020, [Ribose Inc](https://www.ribose.com).
+ * Copyright (c) 2017-2020,2023 [Ribose Inc](https://www.ribose.com).
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification,
@@ -34,35 +34,17 @@
 #include "uniwin.h"
 #endif
 #include <string.h>
-#include <inttypes.h>
+#ifndef __STDC_FORMAT_MACROS
+#define __STDC_FORMAT_MACROS
+#endif
+#include <cinttypes>
 #include <rnp/rnp_def.h>
 #include "types.h"
-#include "crypto.h"
 #include "crypto/mem.h"
 #include "stream-packet.h"
 #include "stream-key.h"
 #include <algorithm>
 
-uint32_t
-read_uint32(const uint8_t *buf)
-{
-    return ((uint32_t) buf[0] << 24) | ((uint32_t) buf[1] << 16) | ((uint32_t) buf[2] << 8) |
-           (uint32_t) buf[3];
-}
-
-uint16_t
-read_uint16(const uint8_t *buf)
-{
-    return ((uint16_t) buf[0] << 8) | buf[1];
-}
-
-void
-write_uint16(uint8_t *buf, uint16_t val)
-{
-    buf[0] = val >> 8;
-    buf[1] = val & 0xff;
-}
-
 size_t
 write_packet_len(uint8_t *buf, size_t len)
 {
@@ -75,7 +57,7 @@ write_packet_len(uint8_t *buf, size_t len)
         return 2;
     } else {
         buf[0] = 0xff;
-        STORE32BE(&buf[1], len);
+        write_uint32(&buf[1], len);
         return 5;
     }
 }
@@ -97,7 +79,7 @@ get_packet_type(uint8_t ptag)
 int
 stream_pkt_type(pgp_source_t &src)
 {
-    if (src_eof(&src)) {
+    if (src.eof()) {
         return 0;
     }
     size_t hdrneed = 0;
@@ -105,7 +87,7 @@ stream_pkt_type(pgp_source_t &src)
         return -1;
     }
     uint8_t hdr[PGP_MAX_HEADER_SIZE];
-    if (!src_peek_eq(&src, hdr, hdrneed)) {
+    if (!src.peek_eq(hdr, hdrneed)) {
         return -1;
     }
     return get_packet_type(hdr[0]);
@@ -116,7 +98,7 @@ stream_pkt_hdr_len(pgp_source_t &src, size_t &hdrlen)
 {
     uint8_t buf[2];
 
-    if (!src_peek_eq(&src, buf, 2) || !(buf[0] & PGP_PTAG_ALWAYS_SET)) {
+    if (!src.peek_eq(buf, 2) || !(buf[0] & PGP_PTAG_ALWAYS_SET)) {
         return false;
     }
 
@@ -190,16 +172,16 @@ get_pkt_len(uint8_t *hdr, size_t *pktlen)
 }
 
 bool
-stream_read_pkt_len(pgp_source_t *src, size_t *pktlen)
+stream_read_pkt_len(pgp_source_t &src, size_t *pktlen)
 {
     uint8_t buf[6] = {};
     size_t  read = 0;
 
-    if (!stream_pkt_hdr_len(*src, read)) {
+    if (!stream_pkt_hdr_len(src, read)) {
         return false;
     }
 
-    if (!src_read_eq(src, buf, read)) {
+    if (!src.read_eq(buf, read)) {
         return false;
     }
 
@@ -212,7 +194,7 @@ stream_read_partial_chunk_len(pgp_source_t *src, size_t *clen, bool *last)
     uint8_t hdr[5] = {};
     size_t  read = 0;
 
-    if (!src_read(src, hdr, 1, &read)) {
+    if (!src->read(hdr, 1, &read)) {
         RNP_LOG("failed to read header");
         return false;
     }
@@ -235,7 +217,7 @@ stream_read_partial_chunk_len(pgp_source_t *src, size_t *clen, bool *last)
     }
     // 2-byte length
     if (hdr[0] < 224) {
-        if (!src_read_eq(src, &hdr[1], 1)) {
+        if (!src->read_eq(&hdr[1], 1)) {
             RNP_LOG("wrong 2-byte length");
             return false;
         }
@@ -243,7 +225,7 @@ stream_read_partial_chunk_len(pgp_source_t *src, size_t *clen, bool *last)
         return true;
     }
     // 4-byte length
-    if (!src_read_eq(src, &hdr[1], 4)) {
+    if (!src->read_eq(&hdr[1], 4)) {
         RNP_LOG("wrong 4-byte length");
         return false;
     }
@@ -256,7 +238,7 @@ bool
 stream_old_indeterminate_pkt_len(pgp_source_t *src)
 {
     uint8_t ptag = 0;
-    if (!src_peek_eq(src, &ptag, 1)) {
+    if (!src->peek_eq(&ptag, 1)) {
         return false;
     }
     return !(ptag & PGP_PTAG_NEW_FORMAT) &&
@@ -267,7 +249,7 @@ bool
 stream_partial_pkt_len(pgp_source_t *src)
 {
     uint8_t hdr[2] = {};
-    if (!src_peek_eq(src, hdr, 2)) {
+    if (!src->peek_eq(hdr, 2)) {
         return false;
     }
     return (hdr[0] & PGP_PTAG_NEW_FORMAT) && (hdr[1] >= 224) && (hdr[1] < 255);
@@ -286,7 +268,7 @@ stream_peek_packet_hdr(pgp_source_t *src, pgp_packet_hdr_t *hdr)
     memset(hdr, 0, sizeof(*hdr));
     if (!stream_pkt_hdr_len(*src, hlen)) {
         uint8_t hdr2[2] = {0};
-        if (!src_peek_eq(src, hdr2, 2)) {
+        if (!src->peek_eq(hdr2, 2)) {
             RNP_LOG("pkt header read failed");
             return RNP_ERROR_READ;
         }
@@ -295,7 +277,7 @@ stream_peek_packet_hdr(pgp_source_t *src, pgp_packet_hdr_t *hdr)
         return RNP_ERROR_BAD_FORMAT;
     }
 
-    if (!src_peek_eq(src, hdr->hdr, hlen)) {
+    if (!src->peek_eq(hdr->hdr, hlen)) {
         RNP_LOG("failed to read pkt header");
         return RNP_ERROR_READ;
     }
@@ -318,7 +300,7 @@ static rnp_result_t
 stream_read_packet_partial(pgp_source_t *src, pgp_dest_t *dst)
 {
     uint8_t hdr = 0;
-    if (!src_read_eq(src, &hdr, 1)) {
+    if (!src->read_eq(&hdr, 1)) {
         return RNP_ERROR_READ;
     }
 
@@ -335,7 +317,7 @@ stream_read_packet_partial(pgp_source_t *src, pgp_dest_t *dst)
 
     while (partlen > 0) {
         size_t read = std::min(partlen, (size_t) PGP_INPUT_CACHE_SIZE);
-        if (!src_read_eq(src, buf, read)) {
+        if (!src->read_eq(buf, read)) {
             free(buf);
             return RNP_ERROR_READ;
         }
@@ -485,6 +467,11 @@ pgp_packet_body_t::pgp_packet_body_t(const uint8_t *data, size_t len)
     secure_ = false;
 }
 
+pgp_packet_body_t::pgp_packet_body_t(const std::vector<uint8_t> &data)
+    : pgp_packet_body_t(data.data(), data.size())
+{
+}
+
 pgp_packet_body_t::~pgp_packet_body_t()
 {
     if (secure_) {
@@ -498,6 +485,12 @@ pgp_packet_body_t::data() noexcept
     return data_.data();
 }
 
+uint8_t *
+pgp_packet_body_t::cur() noexcept
+{
+    return data_.data() + pos_;
+}
+
 size_t
 pgp_packet_body_t::size() const noexcept
 {
@@ -510,6 +503,18 @@ pgp_packet_body_t::left() const noexcept
     return data_.size() - pos_;
 }
 
+void
+pgp_packet_body_t::skip(size_t bt) noexcept
+{
+    pos_ += bt;
+}
+
+void
+pgp_packet_body_t::skip_back(size_t bt) noexcept
+{
+    pos_ = bt > pos_ ? 0 : pos_ - bt;
+}
+
 bool
 pgp_packet_body_t::get(uint8_t &val) noexcept
 {
@@ -554,15 +559,26 @@ pgp_packet_body_t::get(uint8_t *val, size_t len) noexcept
 }
 
 bool
-pgp_packet_body_t::get(pgp_key_id_t &val) noexcept
+pgp_packet_body_t::get(std::vector<uint8_t> &val, size_t len)
 {
-    static_assert(std::tuple_size<pgp_key_id_t>::value == PGP_KEY_ID_SIZE,
-                  "pgp_key_id_t size mismatch");
+    if (pos_ + len > data_.size()) {
+        return false;
+    }
+    val.assign(data_.data() + pos_, data_.data() + pos_ + len);
+    pos_ += len;
+    return true;
+}
+
+bool
+pgp_packet_body_t::get(pgp::KeyID &val) noexcept
+{
+    static_assert(std::tuple_size<pgp::KeyID>::value == PGP_KEY_ID_SIZE,
+                  "pgp::KeyID size mismatch");
     return get(val.data(), val.size());
 }
 
 bool
-pgp_packet_body_t::get(pgp_mpi_t &val) noexcept
+pgp_packet_body_t::get(pgp::mpi &val) noexcept
 {
     uint16_t bits = 0;
     if (!get(bits)) {
@@ -577,13 +593,13 @@ pgp_packet_body_t::get(pgp_mpi_t &val) noexcept
         RNP_LOG("0 mpi");
         return false;
     }
-    if (!get(val.mpi, len)) {
+    val.resize(len);
+    if (!get(val.data(), len)) {
         RNP_LOG("failed to read mpi body");
         return false;
     }
     /* check the mpi bit count */
-    val.len = len;
-    size_t mbits = mpi_bits(&val);
+    size_t mbits = val.bits();
     if (mbits != bits) {
         RNP_LOG(
           "Warning! Wrong mpi bit count: got %" PRIu16 ", but actual is %zu", bits, mbits);
@@ -598,15 +614,15 @@ pgp_packet_body_t::get(pgp_curve_t &val) noexcept
     if (!get(oidlen)) {
         return false;
     }
-    uint8_t oid[MAX_CURVE_OID_HEX_LEN] = {0};
-    if (!oidlen || (oidlen == 0xff) || (oidlen > sizeof(oid))) {
+    if (!oidlen || (oidlen == 0xff)) {
         RNP_LOG("unsupported curve oid len: %" PRIu8, oidlen);
         return false;
     }
+    std::vector<uint8_t> oid(oidlen, 0);
     if (!get(oid, oidlen)) {
         return false;
     }
-    pgp_curve_t res = find_curve_by_OID(oid, oidlen);
+    pgp_curve_t res = pgp::ec::Curve::by_OID(oid);
     if (res == PGP_CURVE_MAX) {
         RNP_LOG("unsupported curve");
         return false;
@@ -693,6 +709,12 @@ pgp_packet_body_t::add(const void *data, size_t len)
     data_.insert(data_.end(), (uint8_t *) data, (uint8_t *) data + len);
 }
 
+void
+pgp_packet_body_t::add(const std::vector<uint8_t> &data)
+{
+    add(data.data(), data.size());
+}
+
 void
 pgp_packet_body_t::add_byte(uint8_t bt)
 {
@@ -711,30 +733,30 @@ void
 pgp_packet_body_t::add_uint32(uint32_t val)
 {
     uint8_t bytes[4];
-    STORE32BE(bytes, val);
+    write_uint32(bytes, val);
     add(bytes, 4);
 }
 
 void
-pgp_packet_body_t::add(const pgp_key_id_t &val)
+pgp_packet_body_t::add(const pgp::KeyID &val)
 {
     add(val.data(), val.size());
 }
 
 void
-pgp_packet_body_t::add(const pgp_mpi_t &val)
+pgp_packet_body_t::add(const pgp::mpi &val)
 {
-    if (!val.len) {
+    if (!val.size()) {
         throw rnp::rnp_exception(RNP_ERROR_BAD_PARAMETERS);
     }
 
-    unsigned idx = 0;
-    while ((idx < val.len - 1) && (!val.mpi[idx])) {
+    size_t idx = 0;
+    while ((idx < val.size() - 1) && (!val[idx])) {
         idx++;
     }
 
-    unsigned bits = (val.len - idx - 1) << 3;
-    unsigned hibyte = val.mpi[idx];
+    size_t  bits = (val.size() - idx - 1) << 3;
+    uint8_t hibyte = val[idx];
     while (hibyte) {
         bits++;
         hibyte = hibyte >> 1;
@@ -742,42 +764,55 @@ pgp_packet_body_t::add(const pgp_mpi_t &val)
 
     uint8_t hdr[2] = {(uint8_t)(bits >> 8), (uint8_t)(bits & 0xff)};
     add(hdr, 2);
-    add(val.mpi + idx, val.len - idx);
+    add(val.data() + idx, val.size() - idx);
 }
 
 void
-pgp_packet_body_t::add_subpackets(const pgp_signature_t &sig, bool hashed)
+pgp_packet_body_t::add_subpackets(const pgp::pkt::Signature &sig, bool hashed)
 {
     pgp_packet_body_t spbody(PGP_PKT_RESERVED);
 
     for (auto &subpkt : sig.subpkts) {
-        if (subpkt.hashed != hashed) {
+        if (subpkt->hashed() != hashed) {
             continue;
         }
 
         uint8_t splen[6];
-        size_t  lenlen = write_packet_len(splen, subpkt.len + 1);
+        size_t  lenlen = write_packet_len(splen, subpkt->data().size() + 1);
         spbody.add(splen, lenlen);
-        spbody.add_byte(subpkt.type | (subpkt.critical << 7));
-        spbody.add(subpkt.data, subpkt.len);
+        spbody.add_byte(subpkt->raw_type() | (subpkt->critical() << 7));
+        spbody.add(subpkt->data().data(), subpkt->data().size());
     }
 
     if (spbody.data_.size() > 0xffff) {
         throw rnp::rnp_exception(RNP_ERROR_BAD_PARAMETERS);
     }
-    add_uint16(spbody.data_.size());
+    switch (sig.version) {
+    case PGP_V4:
+    case PGP_V5:
+        add_uint16(spbody.data_.size());
+        break;
+#if defined(ENABLE_CRYPTO_REFRESH)
+    case PGP_V6:
+        add_uint32(spbody.data_.size());
+        break;
+#endif
+    default:
+        RNP_LOG("should not reach this code");
+        throw rnp::rnp_exception(RNP_ERROR_BAD_STATE);
+    }
     add(spbody.data_.data(), spbody.data_.size());
 }
 
 void
 pgp_packet_body_t::add(const pgp_curve_t curve)
 {
-    const ec_curve_desc_t *desc = get_curve_desc(curve);
+    auto desc = pgp::ec::Curve::get(curve);
     if (!desc) {
         throw rnp::rnp_exception(RNP_ERROR_BAD_PARAMETERS);
     }
-    add_byte((uint8_t) desc->OIDhex_len);
-    add(desc->OIDhex, (uint8_t) desc->OIDhex_len);
+    add_byte((uint8_t) desc->OID.size());
+    add(desc->OID.data(), desc->OID.size());
 }
 
 void
@@ -828,7 +863,7 @@ rnp_result_t
 pgp_packet_body_t::read(pgp_source_t &src) noexcept
 {
     /* Make sure we have enough data for packet header */
-    if (!src_peek_eq(&src, hdr_, 2)) {
+    if (!src.peek_eq(hdr_, 2)) {
         return RNP_ERROR_READ;
     }
 
@@ -837,7 +872,7 @@ pgp_packet_body_t::read(pgp_source_t &src) noexcept
     if (!stream_pkt_hdr_len(src, len)) {
         return RNP_ERROR_BAD_FORMAT;
     }
-    if (!src_peek_eq(&src, hdr_, len)) {
+    if (!src.peek_eq(hdr_, len)) {
         return RNP_ERROR_READ;
     }
     hdr_len_ = len;
@@ -849,7 +884,7 @@ pgp_packet_body_t::read(pgp_source_t &src) noexcept
     }
     tag_ = (pgp_pkt_type_t) ptag;
 
-    if (!stream_read_pkt_len(&src, &len)) {
+    if (!stream_read_pkt_len(src, &len)) {
         return RNP_ERROR_READ;
     }
 
@@ -872,7 +907,7 @@ pgp_packet_body_t::read(pgp_source_t &src) noexcept
     }
 
     size_t read = 0;
-    if (!src_read(&src, data_.data(), len, &read) || (read != len)) {
+    if (!src.read(data_.data(), len, &read) || (read != len)) {
         RNP_LOG("read %d instead of %d", (int) read, (int) len);
         return RNP_ERROR_READ;
     }
@@ -1033,7 +1068,17 @@ pgp_pk_sesskey_t::write(pgp_dest_t &dst) const
 {
     pgp_packet_body_t pktbody(PGP_PKT_PK_SESSION_KEY);
     pktbody.add_byte(version);
-    pktbody.add(key_id);
+#if defined(ENABLE_CRYPTO_REFRESH)
+    if (version == PGP_PKSK_V3) {
+#endif
+        pktbody.add(key_id);
+#if defined(ENABLE_CRYPTO_REFRESH)
+    } else {                             // PGP_PKSK_V6
+        pktbody.add_byte(1 + fp.size()); // A one-octet size of the following two fields.
+        pktbody.add_byte((fp.size() == PGP_FINGERPRINT_V6_SIZE) ? PGP_V6 : PGP_V4);
+        pktbody.add(fp.vec());
+    }
+#endif
     pktbody.add_byte(alg);
     pktbody.add(material_buf.data(), material_buf.size());
     pktbody.write(dst);
@@ -1049,16 +1094,76 @@ pgp_pk_sesskey_t::parse(pgp_source_t &src)
     }
     /* version */
     uint8_t bt = 0;
-    if (!pkt.get(bt) || (bt != PGP_PKSK_V3)) {
+    if (!pkt.get(bt)) {
+        RNP_LOG("Error when reading packet version");
+        return RNP_ERROR_BAD_FORMAT;
+    }
+#if defined(ENABLE_CRYPTO_REFRESH)
+    if ((bt != PGP_PKSK_V3) && (bt != PGP_PKSK_V6)) {
+#else
+    if ((bt != PGP_PKSK_V3)) {
+#endif
         RNP_LOG("wrong packet version");
         return RNP_ERROR_BAD_FORMAT;
     }
-    version = bt;
-    /* key id */
-    if (!pkt.get(key_id)) {
-        RNP_LOG("failed to get key id");
-        return RNP_ERROR_BAD_FORMAT;
+    version = (pgp_pkesk_version_t) bt;
+
+#if defined(ENABLE_CRYPTO_REFRESH)
+    if (version == PGP_PKSK_V3)
+#endif
+    {
+        /* key id */
+        if (!pkt.get(key_id)) {
+            RNP_LOG("failed to get key id");
+            return RNP_ERROR_BAD_FORMAT;
+        }
     }
+#if defined(ENABLE_CRYPTO_REFRESH)
+    else {                          // PGP_PKSK_V6
+        uint8_t fp_and_key_ver_len; // A one-octet size of the following two fields.
+        if (!pkt.get(fp_and_key_ver_len)) {
+            RNP_LOG("Error when reading length of next two fields");
+            return RNP_ERROR_BAD_FORMAT;
+        }
+        if ((fp_and_key_ver_len != 1 + PGP_FINGERPRINT_V4_SIZE) &&
+            (fp_and_key_ver_len != 1 + PGP_FINGERPRINT_V6_SIZE)) {
+            RNP_LOG("Invalid size for key version + length field");
+            return RNP_ERROR_BAD_FORMAT;
+        }
+
+        size_t  fp_len;
+        uint8_t fp_key_version;
+        if (!pkt.get(fp_key_version)) {
+            RNP_LOG("Error when reading key version");
+            return RNP_ERROR_BAD_FORMAT;
+        }
+        switch (fp_key_version) {
+        case 0: // anonymous
+            fp_len = 0;
+            break;
+        case PGP_V4:
+            fp_len = PGP_FINGERPRINT_V4_SIZE;
+            break;
+        case PGP_V6:
+            fp_len = PGP_FINGERPRINT_V6_SIZE;
+            break;
+        default:
+            RNP_LOG("wrong key version used with PKESK v6");
+            return RNP_ERROR_BAD_FORMAT;
+        }
+        if (fp_len && (fp_len + 1 != fp_and_key_ver_len)) {
+            RNP_LOG("size mismatch (fingerprint size and fp+key version length field)");
+            return RNP_ERROR_BAD_FORMAT;
+        }
+        std::vector<uint8_t> vec(fp_len, 0);
+        if (!pkt.get(vec.data(), vec.size())) {
+            RNP_LOG("Error when reading fingerprint");
+            return RNP_ERROR_BAD_FORMAT;
+        }
+        fp = pgp::Fingerprint(vec.data(), vec.size());
+    }
+#endif
+
     /* public key algorithm */
     if (!pkt.get(bt)) {
         RNP_LOG("failed to get palg");
@@ -1066,7 +1171,10 @@ pgp_pk_sesskey_t::parse(pgp_source_t &src)
     }
     alg = (pgp_pubkey_alg_t) bt;
 
-    /* raw signature material */
+    /* symmetric algorithm */
+    salg = PGP_SA_UNKNOWN;
+
+    /* raw encrypted material */
     if (!pkt.left()) {
         RNP_LOG("No encrypted material");
         return RNP_ERROR_BAD_FORMAT;
@@ -1080,103 +1188,39 @@ pgp_pk_sesskey_t::parse(pgp_source_t &src)
     /* we cannot fail here */
     pkt.get(material_buf.data(), material_buf.size());
     /* check whether it can be parsed */
-    pgp_encrypted_material_t material = {};
-    if (!parse_material(material)) {
+    if (!parse_material()) {
         return RNP_ERROR_BAD_FORMAT;
     }
     return RNP_SUCCESS;
 }
 
-bool
-pgp_pk_sesskey_t::parse_material(pgp_encrypted_material_t &material) const
+std::unique_ptr<pgp::EncMaterial>
+pgp_pk_sesskey_t::parse_material() const
 {
-    pgp_packet_body_t pkt(material_buf.data(), material_buf.size());
-    switch (alg) {
-    case PGP_PKA_RSA:
-    case PGP_PKA_RSA_ENCRYPT_ONLY:
-        /* RSA m */
-        if (!pkt.get(material.rsa.m)) {
-            RNP_LOG("failed to get rsa m");
-            return false;
-        }
-        break;
-    case PGP_PKA_ELGAMAL:
-    case PGP_PKA_ELGAMAL_ENCRYPT_OR_SIGN:
-        /* ElGamal g, m */
-        if (!pkt.get(material.eg.g) || !pkt.get(material.eg.m)) {
-            RNP_LOG("failed to get elgamal mpis");
-            return false;
-        }
-        break;
-    case PGP_PKA_SM2:
-        /* SM2 m */
-        if (!pkt.get(material.sm2.m)) {
-            RNP_LOG("failed to get sm2 m");
-            return false;
-        }
-        break;
-    case PGP_PKA_ECDH: {
-        /* ECDH ephemeral point */
-        if (!pkt.get(material.ecdh.p)) {
-            RNP_LOG("failed to get ecdh p");
-            return false;
-        }
-        /* ECDH m */
-        uint8_t bt = 0;
-        if (!pkt.get(bt)) {
-            RNP_LOG("failed to get ecdh m len");
-            return false;
-        }
-        if (bt > ECDH_WRAPPED_KEY_SIZE) {
-            RNP_LOG("wrong ecdh m len");
-            return false;
-        }
-        material.ecdh.mlen = bt;
-        if (!pkt.get(material.ecdh.m, bt)) {
-            RNP_LOG("failed to get ecdh m len");
-            return false;
-        }
-        break;
+    auto enc = pgp::EncMaterial::create(alg);
+    if (!enc) {
+        return nullptr;
     }
-    default:
-        RNP_LOG("unknown pk alg %d", (int) alg);
-        return false;
+#if defined(ENABLE_CRYPTO_REFRESH)
+    enc->version = version;
+#endif
+    pgp_packet_body_t pkt(material_buf);
+    if (!enc->parse(pkt)) {
+        return nullptr;
     }
-
     if (pkt.left()) {
-        RNP_LOG("extra %d bytes in pk packet", (int) pkt.left());
-        return false;
+        RNP_LOG("extra %zu bytes in pk packet", pkt.left());
+        return nullptr;
     }
-    return true;
+    return enc;
 }
 
 void
-pgp_pk_sesskey_t::write_material(const pgp_encrypted_material_t &material)
+pgp_pk_sesskey_t::write_material(const pgp::EncMaterial &material)
 {
     pgp_packet_body_t pktbody(PGP_PKT_PK_SESSION_KEY);
-
-    switch (alg) {
-    case PGP_PKA_RSA:
-    case PGP_PKA_RSA_ENCRYPT_ONLY:
-        pktbody.add(material.rsa.m);
-        break;
-    case PGP_PKA_SM2:
-        pktbody.add(material.sm2.m);
-        break;
-    case PGP_PKA_ECDH:
-        pktbody.add(material.ecdh.p);
-        pktbody.add_byte(material.ecdh.mlen);
-        pktbody.add(material.ecdh.m, material.ecdh.mlen);
-        break;
-    case PGP_PKA_ELGAMAL:
-        pktbody.add(material.eg.g);
-        pktbody.add(material.eg.m);
-        break;
-    default:
-        RNP_LOG("Unknown pk alg: %d", (int) alg);
-        throw rnp::rnp_exception(RNP_ERROR_BAD_PARAMETERS);
-    }
-    material_buf = {pktbody.data(), pktbody.data() + pktbody.size()};
+    material.write(pktbody);
+    material_buf.assign(pktbody.data(), pktbody.data() + pktbody.size());
 }
 
 void
