@@ -32,12 +32,15 @@
 #endif
 #include <errno.h>
 #include <iterator>
+#include "config.h"
 #include "rnp/rnpcfg.h"
 #include "rnpkeys.h"
 #include "defaults.h"
 #include "file-utils.h"
 #include "logging.h"
 
+#define MAX_INPUT_ATTEMPTS 5
+
 /* -----------------------------------------------------------------------------
  * @brief   Reads input from file pointer and converts it securelly to ints
  *          Partially based on ERR34-C from SEI CERT C Coding Standard
@@ -77,6 +80,17 @@ rnp_secure_get_long_from_fd(FILE *fp, long &result, bool allow_empty = true)
     return true;
 }
 
+static bool
+check_attempts(int &attempts)
+{
+    if (attempts >= MAX_INPUT_ATTEMPTS) {
+        printf("Too many attempts. Aborting.\n");
+        return false;
+    }
+    attempts++;
+    return true;
+}
+
 static bool
 is_rsa_keysize_supported(uint32_t keysize)
 {
@@ -119,8 +133,7 @@ ask_curve_name(FILE *input_fp)
     const char *result = NULL;
     int         attempts = 0;
     do {
-        if (attempts >= 10) {
-            printf("Too many attempts. Aborting.\n");
+        if (!check_attempts(attempts)) {
             return NULL;
         }
         printf("Please select which elliptic curve you want:\n");
@@ -133,18 +146,61 @@ ask_curve_name(FILE *input_fp)
         if (ok) {
             result = curves[val - 1];
         }
-        attempts++;
     } while (!ok);
 
     return result;
 }
 
+#if defined(ENABLE_PQC)
+static std::string
+ask_sphincsplus_param_name(FILE *input_fp)
+{
+    std::vector<std::string> params = {"128f", "128s", "192f", "192s", "256f", "256s"};
+    std::vector<std::string> add_info = {"ML-KEM-768 + X25519",
+                                         "ML-KEM-768 + X25519",
+                                         "ML-KEM-1024 + ECDH-NIST-P-384",
+                                         "ML-KEM-1024 + ECDH-NIST-P-384",
+                                         "ML-KEM-1024 + ECDH-NIST-P-384",
+                                         "ML-KEM-1024 + ECDH-NIST-P-384"};
+
+    const size_t pcount = params.size();
+    if (!pcount) {
+        return NULL;
+    }
+    bool        ok = false;
+    std::string result = "";
+    int         attempts = 0;
+    do {
+        if (!check_attempts(attempts)) {
+            return NULL;
+        }
+        printf("Please select which SLH-DSA parameter set you want. In parenthesis, the "
+               "correspondingly chosen encryption subkey is shown:\n");
+        for (size_t i = 0; i < pcount; i++) {
+            printf("\t(%zu) %s (%s)\n", i + 1, params[i].c_str(), add_info[i].c_str());
+        }
+        printf("(default %s)> ", params[0].c_str());
+        long val = 0;
+        ok = rnp_secure_get_long_from_fd(input_fp, val) && (val > 0) && (val <= (long) pcount);
+        if (ok) {
+            result = params[val - 1];
+        }
+    } while (!ok);
+
+    return result;
+}
+#endif
+
 static long
 ask_rsa_bitlen(FILE *input_fp)
 {
     long result = 0;
+    int  attempts = 0;
     do {
         result = DEFAULT_RSA_NUMBITS;
+        if (!check_attempts(attempts)) {
+            return -1;
+        }
         printf("Please provide bit length of the key (between 1024 and 4096):\n(default %d)> ",
                DEFAULT_RSA_NUMBITS);
     } while (!rnp_secure_get_long_from_fd(input_fp, result) ||
@@ -155,13 +211,17 @@ ask_rsa_bitlen(FILE *input_fp)
 static long
 ask_elgamal_bitlen(FILE *input_fp)
 {
+    int attempts = 0;
     do {
+        long result = DEFAULT_ELGAMAL_NUMBITS;
+        if (!check_attempts(attempts)) {
+            return -1;
+        }
         printf(
           "Please provide bit length of the ElGamal key (between %d and %d):\n(default %d) > ",
           ELGAMAL_MIN_P_BITLEN,
           ELGAMAL_MAX_P_BITLEN,
           DEFAULT_ELGAMAL_NUMBITS);
-        long result = DEFAULT_ELGAMAL_NUMBITS;
         if (!rnp_secure_get_long_from_fd(input_fp, result)) {
             continue;
         }
@@ -172,12 +232,17 @@ ask_elgamal_bitlen(FILE *input_fp)
             return result;
         }
     } while (1);
+    return -1;
 }
 
 static long
 ask_dsa_bitlen(FILE *input_fp)
 {
+    int attempts = 0;
     do {
+        if (!check_attempts(attempts)) {
+            return -1;
+        }
         printf(
           "Please provide bit length of the DSA key (between %d and %d):\n(default %d) > ",
           DSA_MIN_P_BITLEN,
@@ -194,21 +259,48 @@ ask_dsa_bitlen(FILE *input_fp)
             return result;
         }
     } while (1);
+
+    return -1;
 }
 
 static bool
 rnpkeys_ask_generate_params(rnp_cfg &cfg, FILE *input_fp)
 {
     long option = 0;
+    int  attempts = 0;
     do {
-        printf("Please select what kind of key you want:\n"
-               "\t(1)  RSA (Encrypt or Sign)\n"
-               "\t(16) DSA + ElGamal\n"
-               "\t(17) DSA + RSA\n" // TODO: See #584
-               "\t(19) ECDSA + ECDH\n"
-               "\t(22) EDDSA + X25519\n"
-               "\t(99) SM2\n"
-               "> ");
+        if (!check_attempts(attempts)) {
+            return false;
+        }
+        printf(
+          "Please select what kind of key you want:\n"
+          "\t(1)  RSA (Encrypt or Sign)\n"
+          "\t(16) DSA + ElGamal\n"
+          "\t(17) DSA + RSA\n" // TODO: See #584
+          "\t(19) ECDSA + ECDH\n"
+#if defined(ENABLE_CRYPTO_REFRESH)
+          "\t(21) EDDSA + ECDH (v6 key) \n"
+          "\t(22) EDDSA + ECDH (v4 key) \n"
+#else
+          "\t(22) EDDSA + ECDH\n"
+#endif
+#if defined(ENABLE_CRYPTO_REFRESH)
+          "\t(23) ED25519 + X25519 (v6 key) \n"
+#endif
+#if defined(ENABLE_PQC)
+          "\t(24) Ed25519Legacy + Curve25519Legacy + (ML-KEM-768 + X25519)\n"
+#if defined(ENABLE_CRYPTO_REFRESH) // PQC primary keys only for v6
+          "\t(25) (ML-DSA-65 + Ed25519) + (ML-KEM-768 + X25519)\n"
+          "\t(27) (ML-DSA-65 + ECDSA-NIST-P-256) + (ML-KEM-768 + ECDH-NIST-P-256)\n"
+          "\t(28) (ML-DSA-87 + ECDSA-NIST-P-384) + (ML-KEM-1024 + ECDH-NIST-P-384)\n"
+          "\t(29) (ML-DSA-65 + ECDSA-brainpoolP256r1) + (ML-KEM-768 + ECDH-brainpoolP256r1)\n"
+          "\t(30) (ML-DSA-87 + ECDSA-brainpoolP384r1) + (ML-KEM-1024 + ECDH-brainpoolP384r1)\n"
+          "\t(31) SLH-DSA-SHA2 + MLKEM-ECDH Composite\n"
+          "\t(32) SLH-DSA-SHAKE + MLKEM-ECDH Composite\n"
+#endif
+#endif
+          "\t(99) SM2\n"
+          "> ");
         if (!rnp_secure_get_long_from_fd(input_fp, option, false)) {
             option = 0;
             continue;
@@ -216,6 +308,9 @@ rnpkeys_ask_generate_params(rnp_cfg &cfg, FILE *input_fp)
         switch (option) {
         case 1: {
             int bits = ask_rsa_bitlen(input_fp);
+            if (bits < 0) {
+                return false;
+            }
             cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_RSA);
             cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_RSA);
             cfg.set_int(CFG_KG_PRIMARY_BITS, bits);
@@ -224,6 +319,9 @@ rnpkeys_ask_generate_params(rnp_cfg &cfg, FILE *input_fp)
         }
         case 16: {
             int bits = ask_dsa_bitlen(input_fp);
+            if (bits < 0) {
+                return false;
+            }
             cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_DSA);
             cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_ELGAMAL);
             cfg.set_int(CFG_KG_PRIMARY_BITS, bits);
@@ -232,6 +330,9 @@ rnpkeys_ask_generate_params(rnp_cfg &cfg, FILE *input_fp)
         }
         case 17: {
             int bits = ask_dsa_bitlen(input_fp);
+            if (bits < 0) {
+                return false;
+            }
             cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_DSA);
             cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_RSA);
             cfg.set_int(CFG_KG_PRIMARY_BITS, bits);
@@ -249,12 +350,100 @@ rnpkeys_ask_generate_params(rnp_cfg &cfg, FILE *input_fp)
             cfg.set_str(CFG_KG_SUBKEY_CURVE, curve);
             break;
         }
+#if defined(ENABLE_CRYPTO_REFRESH)
+        case 21: {
+            cfg.set_str(CFG_KG_V6_KEY, "true");
+            FALLTHROUGH_STATEMENT;
+        }
+#endif
         case 22: {
             cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_EDDSA);
             cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_ECDH);
             cfg.set_str(CFG_KG_SUBKEY_CURVE, "Curve25519");
             break;
         }
+#if defined(ENABLE_CRYPTO_REFRESH)
+        case 23: {
+            cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_ED25519);
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_X25519);
+            cfg.set_str(CFG_KG_V6_KEY, "true");
+            break;
+        }
+#endif
+#if defined(ENABLE_PQC)
+        case 24:
+            cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_EDDSA);
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_ECDH);
+            cfg.set_str(CFG_KG_SUBKEY_CURVE, "Curve25519");
+            cfg.set_str(CFG_KG_SUBKEY_2_ALG, RNP_ALGNAME_KYBER768_X25519);
+            break;
+#if defined(ENABLE_CRYPTO_REFRESH) // PQC primary keys only for v6
+        case 25:
+            cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_DILITHIUM3_ED25519);
+            cfg.set_str(CFG_KG_HASH, RNP_ALGNAME_SHA3_256);
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER768_X25519);
+            cfg.set_str(CFG_KG_V6_KEY, "true");
+            break;
+        case 27:
+            cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_DILITHIUM3_P256);
+            cfg.set_str(CFG_KG_HASH, RNP_ALGNAME_SHA3_256);
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER768_P256);
+            cfg.set_str(CFG_KG_V6_KEY, "true");
+            break;
+        case 28:
+            cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_DILITHIUM5_P384);
+            cfg.set_str(CFG_KG_HASH, RNP_ALGNAME_SHA3_256);
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER1024_P384);
+            cfg.set_str(CFG_KG_V6_KEY, "true");
+            break;
+        case 29:
+            cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_DILITHIUM3_BP256);
+            cfg.set_str(CFG_KG_HASH, RNP_ALGNAME_SHA3_256);
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER768_BP256);
+            cfg.set_str(CFG_KG_V6_KEY, "true");
+            break;
+        case 30:
+            cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_DILITHIUM5_BP384);
+            cfg.set_str(CFG_KG_HASH, RNP_ALGNAME_SHA3_256);
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER1024_BP384);
+            cfg.set_str(CFG_KG_V6_KEY, "true");
+            break;
+        case 31: {
+            std::string param = ask_sphincsplus_param_name(input_fp);
+            if (param == "") {
+                return false;
+            }
+            if (param == "128f" || param == "128s") {
+                cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER768_X25519);
+                cfg.set_str(CFG_KG_HASH, RNP_ALGNAME_SHA256);
+            } else {
+                cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER1024_P384);
+                cfg.set_str(CFG_KG_HASH, RNP_ALGNAME_SHA512);
+            }
+            cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_SPHINCSPLUS_SHA2);
+            cfg.set_str(CFG_KG_PRIMARY_SPHINCSPLUS_PARAM, param);
+            cfg.set_str(CFG_KG_V6_KEY, "true");
+            break;
+        }
+        case 32: {
+            std::string param = ask_sphincsplus_param_name(input_fp);
+            if (param == "") {
+                return false;
+            }
+            if (param == "128f" || param == "128s") {
+                cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER768_X25519);
+                cfg.set_str(CFG_KG_HASH, RNP_ALGNAME_SHA3_256);
+            } else {
+                cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER1024_P384);
+                cfg.set_str(CFG_KG_HASH, RNP_ALGNAME_SHA3_512);
+            }
+            cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_SPHINCSPLUS_SHAKE);
+            cfg.set_str(CFG_KG_PRIMARY_SPHINCSPLUS_PARAM, param);
+            cfg.set_str(CFG_KG_V6_KEY, "true");
+            break;
+        }
+#endif
+#endif
         case 99: {
             cfg.set_str(CFG_KG_PRIMARY_ALG, RNP_ALGNAME_SM2);
             cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_SM2);
@@ -276,7 +465,11 @@ static bool
 rnpkeys_ask_generate_params_subkey(rnp_cfg &cfg, FILE *input_fp)
 {
     long option = 0;
+    int  attempts = 0;
     do {
+        if (!check_attempts(attempts)) {
+            return false;
+        }
         printf("Please select subkey algorithm you want:\n"
                "\t(1)  RSA\n"
                "\t(16) ElGamal\n"
@@ -284,6 +477,14 @@ rnpkeys_ask_generate_params_subkey(rnp_cfg &cfg, FILE *input_fp)
                "\t(18) ECDH\n"
                "\t(19) ECDSA\n"
                "\t(22) EDDSA\n"
+#if defined(ENABLE_PQC)
+               "\t(25) ML-KEN-768 + X25519\n"
+               "\t(26) ML-KEM-1024 + X448\n"
+               "\t(27) ML-KEM-768 + ECDH-NIST-P-256\n"
+               "\t(28) ML-KEM-1024 + ECDH-NIST-P-384\n"
+               "\t(29) ML-KEM-768 + ECDH-brainpoolP256r1\n"
+               "\t(30) ML-KEM-1024 + ECDH-brainpoolP384r1\n"
+#endif
                "\t(99) SM2"
                "> ");
         if (!rnp_secure_get_long_from_fd(input_fp, option, false)) {
@@ -293,18 +494,27 @@ rnpkeys_ask_generate_params_subkey(rnp_cfg &cfg, FILE *input_fp)
         switch (option) {
         case 1: {
             int bits = ask_rsa_bitlen(input_fp);
+            if (bits < 0) {
+                return false;
+            }
             cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_RSA);
             cfg.set_int(CFG_KG_SUBKEY_BITS, bits);
             break;
         }
         case 16: {
             int bits = ask_elgamal_bitlen(input_fp);
+            if (bits < 0) {
+                return false;
+            }
             cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_ELGAMAL);
             cfg.set_int(CFG_KG_SUBKEY_BITS, bits);
             break;
         }
         case 17: {
             int bits = ask_dsa_bitlen(input_fp);
+            if (bits < 0) {
+                return false;
+            }
             cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_DSA);
             cfg.set_int(CFG_KG_SUBKEY_BITS, bits);
             break;
@@ -331,6 +541,26 @@ rnpkeys_ask_generate_params_subkey(rnp_cfg &cfg, FILE *input_fp)
             cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_EDDSA);
             break;
         }
+#if defined(ENABLE_PQC)
+        case 25:
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER768_X25519);
+            break;
+        case 26:
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER1024_X448);
+            break;
+        case 27:
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER768_P256);
+            break;
+        case 28:
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER1024_P384);
+            break;
+        case 29:
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER768_BP256);
+            break;
+        case 30:
+            cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_KYBER1024_BP384);
+            break;
+#endif
         case 99: {
             cfg.set_str(CFG_KG_SUBKEY_ALG, RNP_ALGNAME_SM2);
             if (!cfg.has(CFG_KG_HASH)) {
